@@ -12,6 +12,7 @@ from __future__ import annotations
 
 import asyncio
 from asyncio import Future
+from inspect import isawaitable
 from typing import Any, Awaitable, Callable, Union
 
 
@@ -168,26 +169,32 @@ class Observer:
 
     def __init__(self, callback: Callback, *, loop=None):
         self._queue = [asyncio.Future()]
-        self.callback = callback
+        self._callback = callback
         (loop if loop else asyncio.get_event_loop()).create_task(self._watch())
 
     @property
     def done(self) -> bool:
-        return self._queue[-1].done()
+        return self._queue[0].cancelled()
 
     def stop(self):
-        self._queue[-1].cancel()
+        self._queue[0].cancel()
 
     def push(self, value):
+        if self.done:
+            return
+
+        top = self._queue[-1]
         self._queue.append(asyncio.Future())
-        self._queue[-2].set_result(value)
+        top.set_result(value)
 
     async def _watch(self):
-        while not self._queue[-1].done():
-            future = self._queue[-1]
-            value = await future
-            self._queue.remove(future)
-            self.callback(value)
+        while not self.done:
+            bottom = self._queue[0]
+            value = await bottom
+            self._queue.remove(bottom)
+            result = self._callback(value)
+            if isawaitable(result):
+                await result
 
 
 class ObserverIterator(Observer):
@@ -198,12 +205,12 @@ class ObserverIterator(Observer):
         super().__init__(None, loop=loop)
 
     def stop(self):
-        self._queue[-1].set_exception(StopAsyncIteration)
-        super().stop()
+        self._queue[0].set_exception(StopAsyncIteration)
 
     async def __anext__(self):
-        value = await self._queue[-1]
-        return value
+        if self.done:
+            raise StopAsyncIteration()
+        return await self._queue[0]
 
     async def _watch(self):
         return
